
**Case # 2: Spark Mongo Connector -> Source (File) -> Kafka -> Spark -> MongoDB**<br />

**Story :- To Persist the list of products with timestamp which have been shipped in MongoDB.**<br />

**Algorithm:<br />**
a.) Read data from multiple file(s) which are in different format or from single file (.csv (or) .txt (or) .json) and then publish the data into Kafka Topic.<br />
b.) Spark Structured Streaming as a consumer that connects to this kafka topic with the configured interval (10 secs) and sends the data to Spark job.<br />
c.) Spark job then transforms the data and persist the output (sink) into MongoDB.<br />

**Requirement :- List of Products with status as "Shipped" to be persisted in MongoDB - Real Time Processing<br />**

1.) The source file should contain product information with different status<br />
For Eg:- <br />
        [productId : P1001, productName : Mobile, productPrice : 1000.00, deliveryStatus : Puchased, timestamp] <br />
        [productId : P1001, productName : Mobile, productPrice : 1000.00, deliveryStatus : Shipped, timestamp] <br />
        [productId : P1001, productName : Mobile, productPrice : 1000.00, deliveryStatus : Pending, timestamp] <br />

2.) Then, Spark job should filter the product(s) data based on the status - Shipped.<br />
3.) Once Spark job filters the prodcuts data then persist the output into MongoDB collection named "**product_list_status_shipped**"<br />
    [productId : P1001, productName : Mobile, productPrice : 1000.00, deliveryStatus : Shipped, timestamp]<br />

(com.mongodb.spark.api.java.MongoSpark helper to establish the connectivity from spark to MongoDB)<br />

**Coding Steps:-**<br />
1) Initialize a Spark session object - the primary entry point for Spark application<br />
   SparkSession spark = SparkSession.builder()...
2) Read CSV file<br />
   Dataset<Row> csvData = spark.read()...
3) Read JSON file<br />
   Dataset<Row> jsonData = spark.read()
4) Combine data from both the files (JSON and CSV)<br />
   Dataset<Row> mergedStream = csvData.union(jsonData)
5) Start streaming query to write data to Kafka topic<br />
   mergedStream.selectExpr(...
6) Read data streams as a streaming DataFrame<br />
   Dataset<Row> dataFrame = spark.readStream().format("kafka")...
7) Filtering the fields from dataframe to fetch required data<br />
   Dataset<Row> readWrite = dataFrame.selectExpr("CAST(value AS STRING) AS json")
   .select(functions.from_json(functions.col("json"), schema)
   .as("data"))
   .select("data.*");
8) Filter the DataFrame based on the status and timestamp<br />
   Dataset<Row> filteredData = readWrite.filter(readWrite.col("deliveryStatus")
   .equalTo("Shipped"))
   .filter(readWrite.col("timestamp")
   .geq(startTime)
   .and(dataFrame.col("timestamp")
   .leq(endTime)));
9) Persist the filtered data to MongoDB<br />
   StreamingQuery query = filteredData.writeStream()...<br />
10) Wait for the streaming query to finish<br />
    query.awaitTermination();<br />
11) Catch if any exceptions occur<br />
    e.printStackTrace();

