package com.nisum.spark.dem.Producer;

import lombok.extern.slf4j.Slf4j;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;

@Slf4j
public class StreamingProducer {
    private static String csvFilePath = "src/main/resources/dataformats/FileToKafka.csv";
    private static String jsonFilePath = "src/main/resources/dataformats/FileToKafka.json";
    private static String kafkaBootstrapServers = "localhost:9092";
    private static String kafkaTopic = "TP.WEBFLUX.KAFKA1";

    public static void main(String[] args) {
        log.info("Entered into main method to start StreamingProducer: ");

        SparkSession spark = getSparkSession();

        Dataset<Row> mergedStream = getCombinedData(spark);

        publishToKafka(mergedStream);

        spark.close();
    }

    private static SparkSession getSparkSession() {
        log.info("Initialize SparkSession: ");
        SparkSession spark = SparkSession.builder()
                .appName("DataStreamingWithSparkKafka")
                .master("local[*]")
                .getOrCreate();
        return spark;
    }
    private static Dataset<Row> getCombinedData(SparkSession spark){

        log.info("Read CSV file: ");
        Dataset<Row> csvData = spark.read()
                .format("csv")
                .option("header", "true")
                .load(csvFilePath); // Replace with your CSV file path

        log.info("Read JSON file: ");
        Dataset<Row> jsonData = spark.read()
                .format("json")
                .option("header", "true")
                .load(jsonFilePath); // Replace with your JSON file path

        log.info("Combine data from both the files (JSON and CSV): ");
        Dataset<Row> mergedStream = csvData.select("productId", "productName", "productPrice", "deliveryStatus", "timestamp").union(jsonData.select("productId", "productName", "productPrice", "deliveryStatus", "timestamp"));
        log.debug("mergedStream: ", mergedStream.toJSON());
        mergedStream.show();
        return mergedStream;
    }
    private static void publishToKafka(Dataset<Row> mergedStream){
        log.debug("Start streaming query to write data to Kafka topic: "+kafkaTopic);
        mergedStream.selectExpr("CAST(productId AS STRING) AS key", "to_json(struct(*)) AS value")
                .write()
                .format("kafka")
                .option("kafka.bootstrap.servers", kafkaBootstrapServers)
                .option("topic", kafkaTopic)
                .save();
    }
}
